import 'dart:typed_data';
import 'credentials.dart';
import 'package:shared_preferences/shared_preferences.dart';

enum EncryptMethod { SM2, SECP256K1 }

const String KEY_ADDR = 'key:addr';
const String KEY_PRIVATE = 'key:private';
const String KEY_PUBLIC = 'key:public';

class EncryptoMessage {
  String publicKey;
  Uint8List body;
  EncryptoMessage(this.publicKey, this.body);
}

class CoreIdentity {
  static final CoreIdentity _singleton = new CoreIdentity._internal();

  factory CoreIdentity() {
    return _singleton;
  }

  CoreIdentity._internal();

  Cipher cipher;
  Future<SharedPreferences> _prefs = SharedPreferences.getInstance();
  StringKeypair accountKp;

  void setup(EncryptMethod method) {
    if (method == EncryptMethod.SM2) {
      this.cipher = SM2Cipher();
    }
    if (method == EncryptMethod.SECP256K1) {
      this.cipher = SECPCipher();
    }
  }

  Future init() async {
    final SharedPreferences prefs = await _prefs;
    final addr = prefs.getString(KEY_ADDR) ?? '';
    if (addr.isEmpty) {
      this.accountKp = await this.cipher.makePair();
      prefs.setString(KEY_ADDR, this.accountKp.address);
      prefs.setString(KEY_PUBLIC, this.accountKp.publicKey);
      prefs.setString(KEY_PRIVATE, this.accountKp.privateKey);
    }
  }

  Future restoreAccount(String privateKey) async {
    final SharedPreferences prefs = await _prefs;
    final stringKeypair = await this.cipher.restorePair(privateKey);
    prefs.setString(KEY_ADDR, stringKeypair.address);
    prefs.setString(KEY_PUBLIC, stringKeypair.publicKey);
    prefs.setString(KEY_PRIVATE, stringKeypair.privateKey);
  }

  Future<StringKeypair> account() async {
    final SharedPreferences prefs = await _prefs;

    return StringKeypair(prefs.getString(KEY_ADDR),
        prefs.getString(KEY_PRIVATE), prefs.getString(KEY_PUBLIC));
  }

  Future<EncryptoMessage> sendTo(
      String targetPublickey, Uint8List payload) async {
    final kp = await this.cipher.makePair();

    print('dynamic pri:${kp.privateKey}');
    print('dynamic pub:${kp.publicKey}');

    final body = await this
        .cipher
        .privateEncrypto(kp.privateKey, targetPublickey, payload);
    return EncryptoMessage(kp.publicKey, body.toList());
  }

  Future<Uint8List> decodeMessage(EncryptoMessage message) async {
    final SharedPreferences prefs = await _prefs;
    final privateKey = prefs.getString(KEY_PRIVATE);
    //final privateKey = this.accountKp.privateKey;
    return await this
        .cipher
        .publicDecrypto(privateKey, message.publicKey, message.body);
  }
}
