import 'dart:async';
import 'dart:typed_data';
import 'package:secp256k1cipher/secp256k1cipher.dart' as secpCipher;
import 'package:sm2cipher/sm2cipher.dart' as sm2Cipher;
import 'package:flutter/foundation.dart';

abstract class Cipher {
  Future<StringKeypair> makePair();
  Future<Uint8List> privateEncrypto(
      String privateKey, String publicKey, Uint8List data);
  Future<Uint8List> publicDecrypto(
      String privateKey, String publicKey, Uint8List data);
  Future<StringKeypair> restorePair(String private);
  Future<Uint8List> prcessCrypto(
      Uint8List secret, Uint8List IV, Uint8List data);
}

class StringKeypair {
  String address;
  String privateKey;
  String publicKey;
  StringKeypair(this.address, this.privateKey, this.publicKey);
}

class OperParameter {
  String privateKey;
  String publicKey;
  Uint8List data;
  OperParameter(this.privateKey, this.publicKey, this.data);
}

class CryptoParameter {
  Uint8List secret;
  Uint8List IV;
  Uint8List data;
  CryptoParameter(this.secret, this.IV, this.data);
}

StringKeypair sm2Keypair(Map p) {
  final keypair = sm2Cipher.generateKeyPair();
  final privateKey = sm2Cipher.strinifyPrivateKey(keypair.privateKey);
  final publicKey = sm2Cipher.strinifyPublicKey(keypair.publicKey);
  final address = sm2Cipher.btcAddress(keypair.publicKey);
  return StringKeypair(address, privateKey, publicKey);
}

StringKeypair secpKeypair(Map p) {
  final keypair = secpCipher.generateKeyPair();
  final privateKey = secpCipher.strinifyPrivateKey(keypair.privateKey);
  final publicKey = secpCipher.strinifyPublicKey(keypair.publicKey);
  final address = secpCipher.btcAddress(keypair.publicKey);
  return StringKeypair(address, privateKey, publicKey);
}

StringKeypair sm2Restor(String private) {
  final pri = sm2Cipher.loadPrivateKey(private);
  final public = sm2Cipher.strinifyPublicKeyFromPrivatekey(pri);
  final pub = sm2Cipher.loadPublicKey(public);
  final addr = sm2Cipher.btcAddress(pub);
  return StringKeypair(addr, private, public);
}

StringKeypair secpRestor(String private) {
  final pri = secpCipher.loadPrivateKey(private);
  final public = secpCipher.strinifyPublicKeyFromPrivatekey(pri);
  final pub = secpCipher.loadPublicKey(public);
  final addr = secpCipher.btcAddress(pub);
  return StringKeypair(addr, private, public);
}

Uint8List sm2Encrypto(OperParameter parameter) {
  print(
      'encrypto with private:${parameter.privateKey} pub:${parameter.publicKey} data:${parameter.data}');

  final data = sm2Cipher.pubkeyEncryptRaw(
      parameter.privateKey, parameter.publicKey, parameter.data);
  print('with result:${data['enc']}');
  return data['enc'];
}

Uint8List secpEncrypto(OperParameter parameter) {
  final data = secpCipher.pubkeyEncryptRaw(
      parameter.privateKey, parameter.publicKey, parameter.data);
  return data['enc'];
}

Uint8List sm2Decrypto(OperParameter parameter) {
  return sm2Cipher.privateDecryptRaw(
      parameter.privateKey, parameter.publicKey, parameter.data);
}

Uint8List secpDecrypto(OperParameter parameter) {
  return secpCipher.privateDecryptRaw(
      parameter.privateKey, parameter.publicKey, parameter.data);
}

Uint8List sm2ProcessCrypto(CryptoParameter parameter) {
  return sm2Cipher.processCrypto(
      parameter.secret, parameter.IV, parameter.data);
}

Uint8List secpProcessCrypto(CryptoParameter parameter) {
  return secpCipher.processCrypto(
      parameter.secret, parameter.IV, parameter.data);
}

class SM2Cipher extends Cipher {
  @override
  Future<StringKeypair> makePair() async {
    return await compute<Map, StringKeypair>(sm2Keypair, {});
  }

  @override
  Future<Uint8List> privateEncrypto(
      String privateKey, String publicKey, Uint8List data) async {
    // TODO: implement privateEncrypto
    return await compute<OperParameter, Uint8List>(
        sm2Encrypto, OperParameter(privateKey, publicKey, data));
  }

  @override
  Future<Uint8List> publicDecrypto(
      String privateKey, String publicKey, Uint8List data) async {
    // TODO: implement publicDecrypto
    return await compute<OperParameter, Uint8List>(
        sm2Encrypto, OperParameter(privateKey, publicKey, data));
  }

  @override
  Future<StringKeypair> restorePair(String private) async {
    // TODO: implement restorePair
    return await compute<String, StringKeypair>(sm2Restor, private);
  }

  @override
  Future<Uint8List> prcessCrypto(
      Uint8List secret, Uint8List IV, Uint8List data) async {
    // TODO: implement prcessCrypto
    return await compute<CryptoParameter, Uint8List>(
        sm2ProcessCrypto, CryptoParameter(secret, IV, data));
  }
}

class SECPCipher extends Cipher {
  @override
  Future<StringKeypair> makePair() async {
    // TODO: implement makePair
    return await compute<Map, StringKeypair>(secpKeypair, {});
  }

  @override
  Future<Uint8List> privateEncrypto(
      String privateKey, String publicKey, Uint8List data) async {
    // TODO: implement privateEncrypto
    print("will encrypt:pri:${privateKey} pub:${publicKey} data:${data}");
    final rt_data = await compute<OperParameter, Uint8List>(
        secpEncrypto, OperParameter(privateKey, publicKey, data));
    print("encoded:${rt_data}");
    return rt_data;
  }

  @override
  Future<Uint8List> publicDecrypto(
      String privateKey, String publicKey, Uint8List data) async {
    // TODO: implement publicDecrypto
    return await compute<OperParameter, Uint8List>(
        secpDecrypto, OperParameter(privateKey, publicKey, data));
  }

  @override
  Future<StringKeypair> restorePair(String private) async {
    // TODO: implement restorePair
    return await compute<String, StringKeypair>(secpRestor, private);
  }

  @override
  Future<Uint8List> prcessCrypto(
      Uint8List secret, Uint8List IV, Uint8List data) async {
    // TODO: implement prcessCrypto
    return await compute<CryptoParameter, Uint8List>(
        secpProcessCrypto, CryptoParameter(secret, IV, data));
  }
}
